#ifndef CONFIG_NAP_H
#define CONFIG_NAP_H

/** @file
 *
 * CPU sleeping
 *
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <config/defaults.h>

//#undef		NAP_PCBIOS
//#define		NAP_NULL

#include <config/local/nap.h>

#endif /* CONFIG_NAP_H */
