/*
 * Copyright (C) 2015 Renesas Electronics Corp.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */
#ifndef __DT_BINDINGS_CLOCK_RENESAS_CPG_MSSR_H__
#define __DT_BINDINGS_CLOCK_RENESAS_CPG_MSSR_H__

#define CPG_CORE			0	/* Core Clock */
#define CPG_MOD				1	/* Module Clock */

#endif /* __DT_BINDINGS_CLOCK_RENESAS_CPG_MSSR_H__ */
