// SPDX-License-Identifier: GPL-2.0+
/*
 * Copyright (C) 2018 Álvaro Fernández Rojas <noltari@gmail.com>
 *
 * Derived from linux/drivers/net/ethernet/broadcom/bcm63xx_enet.c
 */

#ifndef __DT_BINDINGS_DMA_BCM6368_H
#define __DT_BINDINGS_DMA_BCM6368_H

#define BCM6368_DMA_ENETSW_RX	0
#define BCM6368_DMA_ENETSW_TX	1

#endif /* __DT_BINDINGS_DMA_BCM6368_H */
