#ifndef _IPXE_RARP_H
#define _IPXE_RARP_H

/** @file
 *
 * Reverse Address Resolution Protocol
 *
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <ipxe/netdevice.h>

extern struct net_protocol rarp_protocol __net_protocol;

#endif /* _IPXE_RARP_H */
