#ifndef _IPXE_EFI_BLACKLIST_H
#define _IPXE_EFI_BLACKLIST_H

/** @file
 *
 * EFI driver blacklist
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

extern void efi_unload_blacklist ( void );

#endif /* _IPXE_EFI_BLACKLIST_H */
