/*
 * This header provides Tegra124-specific constants for binding
 * nvidia,tegra124-car.
 */

#include <dt-bindings/clock/tegra124-car-common.h>

#ifndef _DT_BINDINGS_CLOCK_TEGRA124_CAR_H
#define _DT_BINDINGS_CLOCK_TEGRA124_CAR_H

#define TEGRA124_CLK_PLL_X		227
#define TEGRA124_CLK_PLL_X_OUT0		228

#define TEGRA124_CLK_CCLK_G		262
#define TEGRA124_CLK_CCLK_LP		263

#define TEGRA124_CLK_CLK_MAX		315

#endif	/* _DT_BINDINGS_CLOCK_TEGRA124_CAR_H */
