/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright 2019 Google LLC
 *
 * This provides additional flags used by x86.
 */

#ifndef _DT_BINDINGS_INTERRUPT_CONTROLLER_X86_IRQ_H
#define _DT_BINDINGS_INTERRUPT_CONTROLLER_X86_IRQ_H

#define X86_IRQ_TYPE_SHARED	(1 << 4)
#define X86_IRQ_TYPE_WAKE	(1 << 5)

#endif
