/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) ASPEED Technology Inc.
 */

#define ASPEED_CLK_GATE_ECLK		0
#define ASPEED_CLK_GATE_GCLK		1
#define ASPEED_CLK_GATE_MCLK		2
#define ASPEED_CLK_GATE_VCLK		3
#define ASPEED_CLK_GATE_BCLK		4
#define ASPEED_CLK_GATE_DCLK		5
#define ASPEED_CLK_GATE_LCLK		6
#define ASPEED_CLK_GATE_YCLK		7
#define ASPEED_CLK_GATE_LHCCLK		8
#define ASPEED_CLK_GATE_REFCLK		9
#define ASPEED_CLK_GATE_ESPICLK		10
#define ASPEED_CLK_GATE_UART1CLK	11
#define ASPEED_CLK_GATE_UART2CLK	12
#define ASPEED_CLK_GATE_UART3CLK	13
#define ASPEED_CLK_GATE_UART4CLK	14
#define ASPEED_CLK_GATE_UART5CLK	15
#define ASPEED_CLK_GATE_UART6CLK	16
#define ASPEED_CLK_GATE_UART7CLK	17
#define ASPEED_CLK_GATE_UART8CLK	18
#define ASPEED_CLK_GATE_UART9CLK	19
#define ASPEED_CLK_GATE_UART10CLK	20
#define ASPEED_CLK_GATE_UART11CLK	21
#define ASPEED_CLK_GATE_UART12CLK	22
#define ASPEED_CLK_GATE_UART13CLK	23
#define ASPEED_CLK_GATE_MAC1CLK		24
#define ASPEED_CLK_GATE_MAC2CLK		25
#define ASPEED_CLK_GATE_MAC3CLK		26
#define ASPEED_CLK_GATE_MAC4CLK		27
#define ASPEED_CLK_GATE_RSACLK		28
#define ASPEED_CLK_GATE_SDCLK		29
#define ASPEED_CLK_GATE_SDEXTCLK	30
#define ASPEED_CLK_GATE_EMMCCLK		31
#define ASPEED_CLK_GATE_EMMCEXTCLK	32
#define ASPEED_CLK_GATE_USBUHCICLK	33
#define ASPEED_CLK_GATE_USBPORT1CLK	34
#define ASPEED_CLK_GATE_USBPORT2CLK	35
#define ASPEED_CLK_GATE_FSICLK		36

#define ASPEED_CLK_APLL			37
#define ASPEED_CLK_EPLL			38
#define ASPEED_CLK_DPLL			39
#define ASPEED_CLK_HPLL			40
#define ASPEED_CLK_AHB			41
#define ASPEED_CLK_APB1			42
#define ASPEED_CLK_APB2			43
#define ASPEED_CLK_UART			44
#define ASPEED_CLK_SDIO			45
#define ASPEED_CLK_ECLK			46
#define ASPEED_CLK_ECLK_MUX		47
#define ASPEED_CLK_LHCLK		48
#define ASPEED_CLK_MAC			49
#define ASPEED_CLK_BCLK			50
#define ASPEED_CLK_MPLL			51
#define ASPEED_CLK_24M			52
#define ASPEED_CLK_EMMC			53
#define ASPEED_CLK_UARTX		54
#define ASPEED_CLK_HUARTX		55
