/** @file
 *
 * "bg" keyboard mapping
 *
 * This file is automatically generated; do not edit
 *
 */

FILE_LICENCE ( PUBLIC_DOMAIN );

#include <ipxe/keymap.h>

/** "bg" keyboard mapping */
struct key_mapping bg_mapping[] __keymap = {
};
