#ifndef CONFIG_FDT_H
#define CONFIG_FDT_H

/** @file
 *
 * Flattened Device Tree configuration
 *
 */

FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );

#include <config/defaults.h>

#include <config/local/fdt.h>

#endif /* CONFIG_FDT_H */
