/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2015-2016 Freescale Semiconductor, Inc.
 * Copyright 2017 NXP
 */

#ifndef _PFE_MDIO_H_
#define _PFE_MDIO_H_

int pfe_phy_configure(struct pfe_eth_dev *priv, int dev_id, int phy_id);

#endif /* _PFE_MDIO_H_ */
