/******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation
 * All rights reserved.
 * This program and the accompanying materials
 * are made available under the terms of the BSD License
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/bsd-license.php
 *
 * Contributors:
 *     IBM Corporation - initial implementation
 *****************************************************************************/

#ifndef __XVECT_H
#define __XVECT_H

#define XVECT_M_HANDLER	0x2ff0 /* Master Handler */
#define XVECT_S_HANDLER	0x2ff8 /* Slave Handler  */
#define XVECT_TOPADDR   0x3fff
#define XVECT_SLP_ENTRY SLAVELOOP_LOADBASE

#endif
