/* SPDX-License-Identifier: GPL-2.0+
 *
 * (C) Copyright 2016 Nexell
 * DeokJin, Lee <truevirtue@nexell.co.kr>
 */

#ifndef __ASM_ARCH_NEXELL_GPIO_H
#define __ASM_ARCH_NEXELL_GPIO_H

#include <asm/io.h>
#include <linux/errno.h>

#define PIN_BASE		0

#define MAX_GPIO_BANKS		5

#endif /* __ASM_ARCH_NEXELL_GPIO_H */
