/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2018 Álvaro Fernández Rojas <noltari@gmail.com>
 */

#ifndef __DT_BINDINGS_POWER_DOMAIN_BCM6362_H
#define __DT_BINDINGS_POWER_DOMAIN_BCM6362_H

#define BCM6362_PWR_SAR		0
#define BCM6362_PWR_IPSEC	1
#define BCM6362_PWR_MIPS	2
#define BCM6362_PWR_DECT	3
#define BCM6362_PWR_USBH	4
#define BCM6362_PWR_USBD	5
#define BCM6362_PWR_ROBOSW	6
#define BCM6362_PWR_PCM		7
#define BCM6362_PWR_PERIPH	8
#define BCM6362_PWR_ADSL_PHY	9
#define BCM6362_PWR_GMII_PADS	10
#define BCM6362_PWR_FAP		11
#define BCM6362_PWR_PCIE	12
#define BCM6362_PWR_WLAN_PADS	13

#endif /* __DT_BINDINGS_POWER_DOMAIN_BCM6362_H */
